#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Publishable
#------------------------------------------------------------------------------
translations.define("Publishable.access_restriction",
    ca = u"Restricció d'accés",
    es = u"Restricción de acceso",
    en = u"Access restriction"
)

translations.define("Publishable.access_restriction=None",
    ca = u"Sense restricció",
    es = u"Sin restricción",
    en = u"Unrestricted"
)

# AccessRestriction
#------------------------------------------------------------------------------
translations.define("AccessRestriction",
    ca = u"Restricció d'accés",
    es = u"Restricción de acceso",
    en = u"Access restriction"
)

translations.define("AccessRestriction-plural",
    ca = u"Restriccions d'accés",
    es = u"Restricciones de acceso",
    en = u"Access restrictions"
)

translations.define("AccessRestriction.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("AccessRestriction.restricted_content",
    ca = u"Contingut restringit",
    es = u"Contenido restringido",
    en = u"Restricted content"
)

