#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations


# VimeoVideo
#------------------------------------------------------------------------------
translations.define("VimeoVideo",
    ca = u"Vídeo de Vimeo",
    es = u"Video de Vimeo",
    en = u"Vimeo video"
)

translations.define("VimeoVideo-plural",
    ca = u"Vídeos de Vimeo",
    es = u"Videos de Vimeo",
    en = u"Vimeo videos"
)

translations.define("VimeoVideo.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("VimeoVideo.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("VimeoVideo.video_id",
    ca = u"Identificador del vídeo",
    es = u"Identificador del video",
    en = u"Video ID"
)

# VimeoVideoRenderer
#------------------------------------------------------------------------------
translations.define("VimeoVideoRenderer",
    ca = u"Pintador de fotogrames pels vídeos de Vimeo",
    es = u"Pintador de fotogramas para los videos de Vimeo",
    en = u"Vimeo videos still renderer"
)

translations.define("VimeoVideoRenderer-plural",
    ca = u"Pintadors de fotogrames pels vídeos de Vimeo",
    es = u"Pintadores de fotogramas para los videos de Vimeo",
    en = u"Vimeo videos still renderers"
)

# VideoPlayerSettings
#------------------------------------------------------------------------------
translations.define("VideoPlayerSettings.vimeo",
    ca = u"Vimeo",
    es = u"Vimeo",
    en = u"Vimeo"
)

translations.define("VideoPlayerSettings.vimeo_loop",
    ca = u"Reproduïr continuament",
    es = u"Reproducir continuamente",
    en = u"Loop"
)

translations.define("VideoPlayerSettings.vimeo_allow_fullscreen",
    ca = u"Permetre pantalla completa",
    es = u"Permitir pantalla completa",
    en = u"Allow fullscreen"
)

translations.define("VideoPlayerSettings.vimeo_title",
    ca = u"Mostrar el títol del vídeo",
    es = u"Mostrar el título del video",
    en = u"Show the video's title"
)

translations.define("VideoPlayerSettings.vimeo_byline",
    ca = u"Mostrar informació de l'autor",
    es = u"Mostrar información del autor",
    en = u"Show the video's byline"
)

translations.define("VideoPlayerSettings.vimeo_portrait",
    ca = u"Mostrar imatge de l'autor",
    es = u"Mostrar imagen del autor",
    en = u"Show the author's portrait"
)

translations.define("VideoPlayerSettings.vimeo_color",
    ca = u"Color del reproductor",
    es = u"Color del reproductor",
    en = u"Player color"
)

