#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.controllers.passwordchangecontroller import (
    PasswordChangeBlockController,
    PasswordChangeConfirmationBlockController
)

